Package: gridpattern
Type: Package
Title: 'grid' Pattern Grobs
Version: 1.4.2
Authors@R: c(
        person("Trevor L.", "Davis", role=c("aut", "cre"), email="trevor.l.davis@gmail.com",
               comment = c(ORCID = "0000-0001-6341-4639")),
        person("Mike", "FC", role = "aut", comment = "Code/docs adapted from ggpattern"),
        person("ggplot2 authors", role = "ctb", comment = "some utility functions copied from ggplot2"))
Description: Provides 'grid' grobs that fill in a user-defined area with various patterns.  Includes enhanced versions of the geometric and image-based patterns originally contained in the 'ggpattern' package as well as original 'pch', 'polygon_tiling', 'regular_polygon', 'rose', 'text', 'wave', and 'weave' patterns plus support for custom user-defined patterns.
URL: https://trevorldavis.com/R/gridpattern/,
        https://github.com/trevorld/gridpattern
BugReports: https://github.com/trevorld/gridpattern/issues
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 3.4.0)
Imports: glue, grDevices, grid, memoise, png, rlang, sf, utils
Suggests: ambient, aqp, aRtsy, ggplot2 (>= 3.5.0), gtable, knitr,
        magick (>= 2.7.4), ragg (>= 1.2.0), rmarkdown, scales, svglite
        (>= 2.1.0), testthat, Unicode, vdiffr (>= 1.0.6)
VignetteBuilder: knitr, rmarkdown
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-22 22:48:11 UTC; trevor
Author: Trevor L. Davis [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6341-4639>),
  Mike FC [aut] (Code/docs adapted from ggpattern),
  ggplot2 authors [ctb] (some utility functions copied from ggplot2)
Maintainer: Trevor L. Davis <trevor.l.davis@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-23 10:00:20 UTC
