# Generated by roxygen2: do not edit by hand

S3method(makeContent,alpha_mask)
S3method(makeContent,clipping_path)
S3method(makeContent,pattern)
S3method(print,pattern_hex)
S3method(print,pattern_square)
S3method(print,pattern_weave)
S3method(update_pattern_alpha,GridPattern)
S3method(update_pattern_alpha,GridTilingPattern)
S3method(update_pattern_alpha,default)
S3method(update_pattern_alpha,list)
export(alphaMaskGrob)
export(clippingPathGrob)
export(grid.pattern)
export(grid.pattern_aRtsy)
export(grid.pattern_ambient)
export(grid.pattern_circle)
export(grid.pattern_crosshatch)
export(grid.pattern_fill)
export(grid.pattern_gradient)
export(grid.pattern_image)
export(grid.pattern_magick)
export(grid.pattern_none)
export(grid.pattern_pch)
export(grid.pattern_placeholder)
export(grid.pattern_plasma)
export(grid.pattern_polygon_tiling)
export(grid.pattern_regular_polygon)
export(grid.pattern_rose)
export(grid.pattern_stripe)
export(grid.pattern_text)
export(grid.pattern_wave)
export(grid.pattern_weave)
export(guess_has_R4.1_features)
export(mean_col)
export(names_aRtsy)
export(names_hex)
export(names_magick)
export(names_magick_intensity)
export(names_magick_stripe)
export(names_pattern)
export(names_placeholder)
export(names_polygon_tiling)
export(names_square)
export(names_weave)
export(patternFill)
export(patternGrob)
export(pattern_hex)
export(pattern_square)
export(pattern_weave)
export(reset_image_cache)
export(star_angle)
export(star_scale)
export(update_alpha)
import(grid)
importFrom(glue,glue)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.capabilities)
importFrom(grDevices,dev.capture)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(grDevices,rgb)
importFrom(rlang,"%||%")
importFrom(rlang,abort)
importFrom(rlang,inform)
importFrom(rlang,warn)
importFrom(utils,hasName)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,tail)
